/*
 * Copyright (c) 2015 - 2017, Dries007 & Double Door Development
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions via the Curse or CurseForge platform are not allowed without
 *   written prior approval.
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package net.doubledoordev.mtrm.client;

import joptsimple.internal.Strings;
import net.doubledoordev.mtrm.Helper;
import net.doubledoordev.mtrm.MineTweakerRecipeMaker;
import net.minecraft.client.gui.FontRenderer;
import org.apache.commons.io.FileUtils;

import java.awt.*;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Dries007
 */
public class ClientHelper
{
    public static final int[] BTN_COLORS = {0x505050, 0xAAAAAA, 0xBDC6FF, 0xFF9090};
    public static final int[] TEXT_COLORS = {0xA0A0A0, 0xE0E0E0, 0xFFFFA0, 0xF0F0F0};
    private static File tmpFile;

    private ClientHelper()
    {
    }

    public static List<String> split(FontRenderer renderer, String text, int width)
    {
        return split(renderer, text, width, Integer.MAX_VALUE);
    }

    public static List<String> split(FontRenderer renderer, String text, int width, int lineLimit)
    {
        if (text.length() == 0)
        {
            return new ArrayList<>();
        }
        List<String> list = new ArrayList<>();
        StringBuilder line = new StringBuilder();
        for (int i = 0; i < text.length(); i++)
        {
            char c = text.charAt(i);
            int w = renderer.func_78256_a(line.append(c).toString());
            if (c == '\n')
            {
                list.add(line.substring(0, line.length() - 1));
                if (list.size() >= lineLimit--)
                {
                    return list;
                }
                line = new StringBuilder();
            }
            else if (w >= width)
            {
                list.add(line.substring(0, line.length() - 1) + '→');
                if (list.size() >= lineLimit--)
                {
                    return list;
                }
                line = new StringBuilder();
                line.append(c);
            }
        }
        list.add(line.toString());
        return list;
    }

    public static void drawSplit(FontRenderer renderer, String text, int width, final int color, int x, int y)
    {
        List<String> lines = split(renderer, text, width);
        renderer.field_78295_j = x;
        renderer.field_78296_k = y;
        if (lines.isEmpty())
        {
            return;
        }
        renderer.func_78276_b(lines.remove(0), x, y, color);
        for (String line : lines)
        {
            renderer.field_78296_k += renderer.field_78288_b;
            renderer.field_78295_j = x;
            renderer.func_78255_a(line, false);
        }
    }

    public static void download()
    {
        try
        {
            tmpFile = File.createTempFile(Helper.MODID + "_", ".zs");

            // todo: actually download
            FileUtils.copyFile(Helper.getScriptFile(), tmpFile);

            String ee = MineTweakerRecipeMaker.getExternalEditor();

            if (Strings.isNullOrEmpty(ee))
            {
                Desktop.getDesktop().open(tmpFile);
            }
            else
            {
                Runtime.getRuntime().exec(ee.replace("%f", tmpFile.getAbsolutePath()));
            }
        }
        catch (Exception e)
        {
            MineTweakerRecipeMaker.log().error("Can't download & edit.", e);
        }
    }

    public static void upload()
    {
        try
        {
            MineTweakerRecipeMaker.log().info(FileUtils.readFileToString(tmpFile, "UTF-8"));

            tmpFile.delete();
        }
        catch (Exception e)
        {
            MineTweakerRecipeMaker.log().error("Can't upload! Changes may be lost!", e);
        }
    }
}
