/*
 * Copyright (c) 2015 - 2017, Dries007 & Double Door Development
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions via the Curse or CurseForge platform are not allowed without
 *   written prior approval.
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package net.doubledoordev.mtrm.client;

import net.doubledoordev.mtrm.MineTweakerRecipeMaker;
import net.doubledoordev.mtrm.client.elements.GuiElement;
import net.doubledoordev.mtrm.client.elements.StringElement;
import net.doubledoordev.mtrm.xml.Function;
import net.doubledoordev.mtrm.xml.XmlParser;
import net.minecraft.client.gui.GuiYesNo;
import net.minecraft.client.gui.inventory.GuiContainer;

/**
 * @author Dries007
 */
public class GuiFunction extends GuiListBase implements GuiElement.GuiElementCallback
{
    protected static final int ID_SAVE = 10;
    private final GuiContainer parent;
    private final Function function;
    private final String genericText;
    private String currentText;

    public GuiFunction(GuiContainer parent, Function function)
    {
        super(parent.field_147002_h);
        this.parent = parent;
        this.function = function;
        StringBuilder textBuilder = new StringBuilder();
        for (Object obj : function.parts)
        {
            if (obj instanceof String)
            {
                textBuilder.append(obj);
                guiElements.add(new StringElement(this, (String) obj));
            }
            else
            {
                XmlParser.IStringObject sObj = (XmlParser.IStringObject) obj;
                textBuilder.append(sObj.toHumanText());
                GuiElement e = sObj.toGuiElement(this);
                if (e != null)
                {
                    guiElements.add(e);
                }
                else
                {
                    guiElements.add(new StringElement(this, sObj.toHumanText(), 0xFF0000));
                }
            }
            textBuilder.append(' ');
        }
        genericText = textBuilder.toString();
        currentText = "";
    }

    @Override
    public void func_73876_c()
    {
        super.func_73876_c();
        StringBuilder sb = new StringBuilder();
        for (GuiElement obj : guiElements)
        {
            sb.append(obj.save());
        }
        currentText = sb.toString();
    }

    @Override
    public void func_73866_w_()
    {
        super.func_73866_w_();
        changes = true;
    }

    @Override
    protected void func_146979_b(int mouseX, int mouseY)
    {
        // todo: remove
        // debug start
        field_146289_q.func_78276_b("Generic", -200, -100, 0xFF5050);
        field_146289_q.func_78279_b(genericText, -200, -90, 200, 0xFFFFFF);
        field_146289_q.func_78276_b("Current", field_146999_f + 25, -100, 0xFF5050);
        field_146289_q.func_78279_b(currentText, field_146999_f + 25, -90, 200, 0xFFFFFF);
        // debug end

        super.func_146979_b(mouseX, mouseY);
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        super.func_146976_a(partialTicks, mouseX, mouseY);
        field_146289_q.func_78276_b(function.name, guiLeft + field_146999_f / 2 - field_146289_q.func_78256_a(function.name) / 2, guiTop - 10, 0xFFFFFF);
    }

    @Override
    protected void exit()
    {
        this.field_146297_k.func_147108_a(parent);
    }

    @Override
    protected void ok()
    {
        this.field_146297_k.func_147108_a(new GuiYesNo(this, "Does this look good?", currentText, ID_SAVE));
    }

    @Override
    public void func_73878_a(boolean result, int id)
    {
        switch (id)
        {
        case ID_SAVE:
            if (result)
            {
                confirm_ok();
            }
            else
            {
                this.field_146297_k.func_147108_a(this);
            }
            break;
        default:
            super.func_73878_a(result, id);
        }
    }

    private void confirm_ok()
    {
        //todo: save currentText
        // Serverside:
        // do packet stuff
        // figure out place in file
        // add author + timestamp
        MineTweakerRecipeMaker.log().info("Saved: {}", currentText);
        this.field_146297_k.func_147108_a(parent);
    }
}
