/*
 * Copyright (c) 2015 - 2017, Dries007 & Double Door Development
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * + Redistributions via the Curse or CurseForge platform are not allowed without
 *   written prior approval.
 *
 * + Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * + Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package net.doubledoordev.mtrm.client.parts;

import com.google.common.collect.Lists;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.Gui;
import net.minecraft.client.gui.GuiTextField;

import java.util.List;

/**
 * @author Dries007
 */
public class GuiMultilineTextField extends GuiTextField
{
    private final FontRenderer fr;
    private List<String> lines = Lists.newArrayList();
    private float scroll = 0;

    public GuiMultilineTextField(int componentId, FontRenderer fr, int x, int y, int par5Width, int par6Height)
    {
        super(componentId, fr, x, y, par5Width, par6Height);
        this.fr = fr;
        func_146184_c(true);
        func_146203_f(Integer.MAX_VALUE / 2);
    }

    private void recomputeLines()
    {
        lines = fr.func_78271_c(field_146216_j, field_146218_h - 8);
        int chars = 0;
        for (int i = 0; i < lines.size(); i++)
        {
            String line = lines.get(i);
            chars += line.length();
            if (field_146216_j.length() > chars && field_146216_j.charAt(chars) == ' ')
            {
                lines.set(i, line + ' ');
                chars++;
            }
        }
        if (!needsScrolling())
        {
            setScroll(0);
        }
    }

    public boolean needsScrolling()
    {
        return lines.size() > (field_146219_i - (field_146215_m ? 8 : 0)) / fr.field_78288_b;
    }

    public void setScroll(float scroll)
    {
        this.scroll = scroll;
    }

    @Override
    public void func_146180_a(String textIn)
    {
        super.func_146180_a(textIn);
        recomputeLines();
    }

    @Override
    public void func_146191_b(String textToWrite)
    {
        super.func_146191_b(textToWrite);
        recomputeLines();
    }

    @Override
    public void func_146175_b(int num)
    {
        super.func_146175_b(num);
        recomputeLines();
    }

    public void func_146194_f()
    {
        if (!field_146220_v)
        {
            return;
        }

        if (field_146215_m)
        {
            func_73734_a(field_146209_f - 1, field_146210_g - 1, field_146209_f + field_146218_h + 1, field_146210_g + field_146219_i + 1, -6250336);
            func_73734_a(field_146209_f, field_146210_g, field_146209_f + field_146218_h, field_146210_g + field_146219_i, -16777216);
        }

        final int txtColor = field_146215_m ? field_146222_t : field_146221_u;
        int x = field_146215_m ? field_146209_f + 4 : field_146209_f;
        int y = field_146215_m ? field_146210_g + 4 : field_146210_g;

        y -= (int) (scroll * (lines.size() - 1)) * fr.field_78288_b;

        int cursorPosition = this.field_146224_r;
        int selectionEnd = this.field_146223_s;
        if (selectionEnd < cursorPosition)
        {
            selectionEnd = this.field_146224_r;
            cursorPosition = this.field_146223_s;
        }

        int chars = 0;
        boolean selected = false;
        for (int i = 0; i < lines.size(); i++)
        {
            final String line = lines.get(i);
            final int length = line.length();

            boolean render = y > field_146210_g && y < field_146210_g + field_146219_i - fr.field_78288_b;

            int subSelectionStart = 0;

            boolean startSelection = cursorPosition >= chars && cursorPosition <= chars + length;

            if (startSelection)
            {
                subSelectionStart = cursorPosition - chars;
                selected = true;
            }

            if (selected)
            {
                int subSelectionEnd = Math.min(length, selectionEnd - chars);
                if (subSelectionEnd < length)
                {
                    selected = false;
                }

                int offsetL = field_146211_a.func_78256_a(line.substring(0, subSelectionStart));
                int offsetR = field_146211_a.func_78256_a(line.substring(0, subSelectionEnd));

                if (render)
                {
                    if (subSelectionEnd != subSelectionStart)
                    {
                        Gui.func_73734_a(x + offsetL, y, x + offsetR, y + 1 + this.field_146211_a.field_78288_b, -3092272);
                        field_146211_a.func_175063_a(line.substring(0, subSelectionStart), x, y, txtColor);
                        field_146211_a.func_175063_a(line.substring(subSelectionStart, subSelectionEnd), x + offsetL, y, txtColor);
                        field_146211_a.func_175063_a(line.substring(subSelectionEnd, length), x + offsetR, y, txtColor);
                    }
                    else
                    {
                        field_146211_a.func_175063_a(line, x, y, txtColor);
                    }
                }
            }
            else if (render)
            {
                field_146211_a.func_175063_a(line, x, y, txtColor);
            }

            int offsetL = field_146211_a.func_78256_a(line.substring(0, subSelectionStart));
            if (render && startSelection && (subSelectionStart != 0 || i == 0) && field_146213_o && field_146214_l / 6 % 2 == 0)
            {
                this.func_73728_b(x + offsetL, y - 1, y + 1 + this.field_146211_a.field_78288_b, 0xFFFF0000);
            }

            chars += length;
            y += fr.field_78288_b;
        }
    }

    @Override
    public void func_146203_f(int length)
    {
        super.func_146203_f(length);
        recomputeLines();
    }
}
